﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.CodeDom.Compiler;

namespace GTTCore.Fonts
{
    class FontBuilder
    {
        private string _Input;
        private string _BaseName;

        public FontBuilder(string input)
        {
            _Input = input;
            _BaseName = input.Substring(0, _Input.IndexOf("."));
        }



        private bool GenerateTinyFont()
        {
            Console.WriteLine("Generating TinyFont for {0}", _Input);
            string Parm = string.Format("{0} " + TinyFontFileName , _Input);
            ProcessStartInfo si = new ProcessStartInfo("extra\\TFConvert.exe", Parm);
            si.WindowStyle = ProcessWindowStyle.Hidden;
            Process p = Process.Start(si);
            p.WaitForExit();
            if (p.ExitCode != 0)
            {
                Console.WriteLine("Error");
                return false;
            }
            return true;
        }

        private bool PatchFont()
        {
            Console.WriteLine("Patching font");
            byte[] dat = File.ReadAllBytes(TinyFontFileName);
            dat[0x14] = 0xfa;
            dat[0x15] = 0x4f;
            File.WriteAllBytes(TinyFontFileName, dat);
            return true;
        }



        private bool CompileFont()
        {
            Console.WriteLine("Compiling font to dll");
            CodeDomProvider provider = CodeDomProvider.CreateProvider("CSharp");
            CompilerParameters param = new CompilerParameters();
            string[] files = { "extra\\Resources.Designer.cs", "extra\\AssemblyInfo.cs" };
            FileInfo fi = new FileInfo("extra\\AssemblyInfo.cs");
            param.OutputAssembly = DllName;
            param.CompilerOptions = "/noconfig /nowarn:1701,1702 /nostdlib+ /errorreport:prompt /warn:4 /define:DEBUG;TRACE  /debug+ /debug:full /optimize- /resource:extra\\FontTemplate.Properties.Resources.resources";
            param.ReferencedAssemblies.Add("extra\\Microsoft.SPOT.Graphics.dll");
            param.ReferencedAssemblies.Add("extra\\Microsoft.SPOT.Native.dll");
            param.ReferencedAssemblies.Add("extra\\mscorlib.dll");
            CompilerResults res = provider.CompileAssemblyFromFile(param, files);
            foreach (CompilerError s in res.Errors)
            {
                Console.WriteLine(s.ToString());
            }
            return res.Errors.Count == 0;
        }

        private static string TinyFontFileName
        {
            get
            {
                return Path.GetTempPath() + "Font.tinyfont";
            }
        }

        private static string DllName
        {
            get
            {
                return Path.GetTempPath() + "FontTemplate.dll";
            }
        }

        private static string TinyResourceFileName
        {
            get
            {
                return Path.GetTempPath() + "FontTemplate.Properties.Resources.tinyresources";
            }
        }
        private bool GeneratePE(string filename)
        {
            if (File.Exists(TinyResourceFileName))
                File.Delete(TinyResourceFileName);
            File.Copy(TinyFontFileName, TinyResourceFileName);
            string app = "extra\\MetaDataProcessor.exe";
            string parm = "-loadHints extra\\Microsoft.SPOT.Graphics extra\\Microsoft.SPOT.Graphics.dll -loadHints extra\\Microsoft.SPOT.Native extra\\Microsoft.SPOT.Native.dll -loadHints mscorlib extra\\mscorlib.dll -parse " + DllName + " -minimize -importResource "+TinyResourceFileName+" -compile " + filename;
            ProcessStartInfo si = new ProcessStartInfo(app, parm);
            si.WindowStyle = ProcessWindowStyle.Hidden;
            Process p = Process.Start(si);
            p.WaitForExit();
            return p.ExitCode == 0;
        }

        public bool ProcessFont(string filename)
        {
            
            if (!GenerateTinyFont())
                return false;
            
            if (!PatchFont())
                return false;
            
            if (!CompileFont())
                return false;
            
            if (!GeneratePE(filename))
                return false;
    
            return true;
        }

    }

}
